AutomaticWateringSystem = {}; 

local function AddTextAppend(Oldtext, Newtext)
    local Append = TextAppend.new();
    Append.Oldtext = Oldtext;
    Append.Newtext = Newtext;
    TextAppendList[table.getn(TextAppendList) + 1] = Append;
end
local function ClearTextAppend()
    for i = 0, table.getn(TextAppendList) do TextAppendList[i] = nil; end
end
TextAppendList = {}
TextAppend = {Oldtext, Newtext}
TextAppend.new = function()
    local self = {}
    return self;
end;

function RoundValue(num, numDecimalPlaces)
	local mult = 10^(numDecimalPlaces or 0)
	return math.floor(num * mult + 0.5) / mult
  end

SaveList = {};

local function GeneratePlaceableHash(CurrentPlaceable)
	local Hash = "";
	Hash = Hash..tostring(RoundValue(CurrentPlaceable.position.x,5))..tostring(RoundValue(CurrentPlaceable.position.y,5))..tostring(RoundValue(CurrentPlaceable.position.z,5));
	Hash = Hash..tostring(RoundValue(CurrentPlaceable.rotation.x,5))..tostring(RoundValue(CurrentPlaceable.rotation.y,5))..tostring(RoundValue(CurrentPlaceable.rotation.z,5));
	Hash = Hash:gsub("%D+", "");
	return Hash;
end;

local function IsInSaveList(PlaceableHash)
    local Found = false;
    for i = 0, table.getn(SaveList) do
        if (SaveList[i] ~= nil) then
            if (SaveList[i] == PlaceableHash) then
                Found = true;
                break
            end
        end
    end
    return Found;
end;

local LastPlaceableUiHash = nil;

local function DisableWatering(self)

	local CurrentHash = GeneratePlaceableHash(self.selectedProductionPoint.owningPlaceable);

	for i=0,table.getn(SaveList) do
		if SaveList[i] ~= nil then 
			if CurrentHash == SaveList[i] then SaveList[i] = nil; end;
		end;
	end;

	LastPlaceableUiHash = nil;
	self.storageList:reloadData();
end
local function EnableWatering(self)
		SaveList[table.getn(SaveList)+1] = GeneratePlaceableHash(self.selectedProductionPoint.owningPlaceable);
		AutomaticWateringSystem:hourChanged();
		LastPlaceableUiHash = nil;
		self.storageList:reloadData();
end
local function UpdateMenuButtons(self)
	local CurrentHash = GeneratePlaceableHash(self.selectedProductionPoint.owningPlaceable);
	if LastPlaceableUiHash == nil or LastPlaceableUiHash ~= CurrentHash then
        ClearTextAppend();
		if IsInSaveList(CurrentHash) then
			local OrginalText = g_fillTypeManager:getFillTypeByIndex(FillType.WATER).title;
			local TextToSet = OrginalText.." ("..g_i18n:getText("AutomaticWatering_Enabled")..")";
			 AddTextAppend(OrginalText,TextToSet);
		end;
		--if not IsInSaveList(CurrentHash) then AddTextAppend(g_fillTypeManager:getFillTypeByIndex(FillType.WATER).title,"NOT"); end;
        LastPlaceableUiHash = CurrentHash;
        self.storageList:reloadData();
        return;
    end

    local isProductionListActive = self.productionList == FocusManager:getFocusedElement()
    local fillType, isInput = self:getSelectedStorageFillType()
	
	if (not isProductionListActive and fillType == FillType.WATER and isInput) or isProductionListActive then
        if IsInSaveList(CurrentHash) then
			table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.UNLOAD,
				text = g_i18n:getText("Disable_AutomaticWatering"),
				callback = function()
					g_gui:showYesNoDialog({
						text = g_i18n:getText("Disable_AutomaticWatering_Message"),
						title = g_i18n:getText("Disable_AutomaticWatering_Title"),
						callback = function(yes)
							if yes then DisableWatering(self) end;
						end
					});
				end;
			})
        end

		if not IsInSaveList(CurrentHash) then
			table.insert(self.menuButtonInfo, {
				profile = "buttonOk",
				inputAction = InputAction.UNLOAD,
				text = g_i18n:getText("Enable_AutomaticWatering"),
				callback = function()
					g_gui:showYesNoDialog({
						text = g_i18n:getText("Enable_AutomaticWatering_Message"),
						confirmText = g_i18n:getText("Enable_AutomaticWatering_Title"),
						callback = function(yes)
							if yes then EnableWatering(self) end;
						end
					});
				end;
			})
        end

        self:setMenuButtonInfoDirty()
    end
end;

function AutomaticWateringSystem:hourChanged()
	for i=#g_currentMission.placeableSystem.placeables, 1, -1 do
        local CurrentPlaceable = g_currentMission.placeableSystem.placeables[i];
		local CurrentHash = GeneratePlaceableHash(CurrentPlaceable);

		if IsInSaveList(CurrentHash) then 
			local CurrentFillLevel = CurrentPlaceable.spec_productionPoint.productionPoint.storage:getFillLevel(FillType.WATER);
			local FreeCapacity = CurrentPlaceable.spec_productionPoint.productionPoint.storage:getFreeCapacity(FillType.WATER);
			local WaterPricePerLiter = g_fillTypeManager:getFillTypeByIndex(FillType.WATER).pricePerLiter;
			local Price = WaterPricePerLiter * FreeCapacity * g_currentMission.missionInfo.buyPriceMultiplier * 2
			if FreeCapacity > 0 then
				print("CAP: "..FreeCapacity.."  PRICE:"..Price);
				g_currentMission:addMoney(-Price, CurrentPlaceable:getOwnerFarmId(), MoneyType.PRODUCTION_COSTS, true);
				CurrentPlaceable.spec_productionPoint.productionPoint.storage:setFillLevel(CurrentFillLevel+FreeCapacity, FillType.WATER);
			end;
		end;

    end
end

function AutomaticWateringSystem:saveSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex);
	end;
	savegameFolderPath = savegameFolderPath.."/"
	local key = "AutomaticWateringSystem";
	local storePlace = g_currentMission.storeSpawnPlaces[1];
	local xmlFile = createXMLFile(key, savegameFolderPath.."AutomaticWateringSystem.xml", key);

	if table.getn(SaveList) == 0 then setXMLString(xmlFile, key.."#SaveList", ""); end;

	if table.getn(SaveList) > 0 then
		local SaveString = "";

		for i=0,table.getn(SaveList) do
			if SaveList[i] ~= nil then 
				SaveString = SaveString..SaveList[i]..",";
			end;
		end;

		SaveString = SaveString:sub(1, -2);
		setXMLString(xmlFile, key.."#SaveList", SaveString);
	end;

	saveXMLFile(xmlFile);
	delete(xmlFile);
end;
function AutomaticWateringSystem:loadSettings()
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), g_currentMission.missionInfo.savegameIndex);
	end;
	savegameFolderPath = savegameFolderPath.."/"
	local key = "AutomaticWateringSystem";

	if fileExists(savegameFolderPath.."AutomaticWateringSystem.xml") then
		local xmlFile = loadXMLFile(key, savegameFolderPath.."AutomaticWateringSystem.xml");
		local SaveString = getXMLString(xmlFile, key.."#SaveList");

		if string.len(SaveString) > 0 then
			for word in string.gmatch(SaveString, '([^,]+)') do
				SaveList[table.getn(SaveList)+1] = word;
			end
		end;
		delete(xmlFile);
	end;
end;

function AutomaticWateringSystem:loadMap(name)	
	AutomaticWateringSystem:loadSettings();
	g_messageCenter:subscribe(MessageType.HOUR_CHANGED, self.hourChanged, self)
end;

function AutomaticWateringSystem:onSell()
	local spec = self.spec_productionPoint
	local CurrentHash = GeneratePlaceableHash(spec.productionPoint.owningPlaceable);

	for i=0,table.getn(SaveList) do
		if SaveList[i] ~= nil then 
			if CurrentHash == SaveList[i] then SaveList[i] = nil; end;
		end;
	end;
end
function AutomaticWateringSystem.registerEventListeners(placeableType)
    SpecializationUtil.registerEventListener(placeableType, "onSell", AutomaticWateringSystem)
end

function SetTextInternal(self)
    for i = 0, table.getn(TextAppendList) do
        if (TextAppendList[i] ~= nil and TextAppendList[i].Oldtext == self.text) then
            self.text = TextAppendList[i].Newtext;
            break
        end
    end
end
local function OnPageChange(self)
    LastPlaceableUiHash = nil;
    ClearTextAppend();
end
local function OnClose(self)
	LastPlaceableUiHash = nil;
    ClearTextAppend();
end

PlaceableProductionPoint.registerEventListeners = Utils.appendedFunction(PlaceableProductionPoint.registerEventListeners, AutomaticWateringSystem.registerEventListeners)
InGameMenuProductionFrame.updateMenuButtons = Utils.appendedFunction(InGameMenuProductionFrame.updateMenuButtons, UpdateMenuButtons);
FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, AutomaticWateringSystem.saveSettings);
TextElement.setTextInternal = Utils.appendedFunction(TextElement.setTextInternal, SetTextInternal);
TabbedMenu.onPageChange = Utils.appendedFunction(TabbedMenu.onPageChange,OnPageChange);
TabbedMenu.onClose = Utils.appendedFunction(TabbedMenu.onClose,OnClose);
addModEventListener(AutomaticWateringSystem);